@@ -4,5 +4,5 @@ Module for keeping exhibit-wide constants
 
 ORIGINAL_VALUES_DB = "Number of unique values is above category threshold"
 ORIGINAL_VALUES_PAIRED = "See paired column"
-MISSING_DATA_STR = "Missing data"
-UUID_PLACEHOLDER = "uuid_placeholder"
+ORIGINAL_VALUES_REGEX = "regex"
+MISSING_DATA_STR = "Missing data"
\ No newline at end of file
