@@ -37,6 +37,7 @@ class ConstraintHandler:
         self.rng = spec_dict["_rng"]
         self.dependent_column = None
         self.independent_expression = None
+        self.current_action = None
         self.input = anon_df
         self.output = None
 
@@ -133,13 +134,19 @@ class ConstraintHandler:
         output_df = source.copy()
 
         dispatch_dict = {
-            "make_outlier"    : self.make_outlier,
-            "sort_ascending"  : partial(self.sort_values, ascending=True),
-            "sort_descending" : partial(self.sort_values, ascending=False),
-            "make_distinct"   : self.make_distinct,
-            "make_same"       : self.make_same,
+            "make_outlier"         : self.make_outlier,
+            "sort_ascending"       : partial(self.sort_values, asc=True),
+            "sort_descending"      : partial(self.sort_values, asc=False),
+            "make_distinct"        : self.make_distinct,
+            "make_same"            : self.make_same,
             "generate_as_sequence" : self.generate_as_sequence,
-            "geo_make_regions" : geo_make_regions,
+            "generate_as_repeating_sequence" :
+                        partial(self.generate_as_sequence, repeating=True),
+            "geo_make_regions"     : geo_make_regions,
+            "sort_and_skew_right"  : partial(self.sort_and_skew, direction="right"),
+            "sort_and_skew_left"   : partial(self.sort_and_skew, direction="left"),
+            "sort_and_make_peak"   : partial(self.sort_and_skew, direction="up"),
+            "sort_and_make_valley" : partial(self.sort_and_skew, direction="down"),
         }
 
         kwargs_dict = {
@@ -164,23 +171,25 @@ class ConstraintHandler:
             #boolean condition is a FILTER so row matching it must be adjusted
             if output_df[cc_filter_mask].empty:
                 continue
+            
+            # target_str can either be a single column or a comma separater list
+            # each action_func decides how it wants to handle the targets: either
+            # act on both at the same time or sequentially, one at a time.
+            for target_str, action_str in cc_targets.items():
 
-            for target_col, action_str in cc_targets.items():
-                if (action_func := dispatch_dict.get(action_str, None)):
+                actions = [x.strip() for x in action_str.split(",")]
+                
+                for action in actions:
+                    self.current_action = action
+                    if (action_func := dispatch_dict.get(action, None)):
 
-                    _kwargs = kwargs_dict.get(action_str, {})
+                        _kwargs = kwargs_dict.get(action, {})
 
-                    # geospatial constaints act on two columns (latitude and longitude)
-                    if action_str[:3] == "geo":
-                        output_df = action_func(
-                        output_df, cc_filter_idx, target_col, cc_partitions, **_kwargs)
-                        continue
+                        # overwrite the original DF row IDs with the adjusted ones
+                        output_df.loc[cc_filter_idx] = action_func(
+                            output_df, cc_filter_idx, target_str,
+                            cc_partitions, **_kwargs)
 
-                    # overwrite the original target column with the adjusted one
-                    output_df.loc[cc_filter_idx, target_col] = action_func(
-                        output_df, cc_filter_idx, target_col, cc_partitions, **_kwargs)
-
-        
         return output_df
 
     def adjust_dataframe_to_fit_constraint(self, anon_df, bool_constraint):
@@ -194,12 +203,6 @@ class ConstraintHandler:
         '''
 
         output_df = anon_df.copy()
-        num_cols = self.spec_dict["metadata"]["numerical_columns"]
-        for num_col in num_cols:
-            # derived columns are in metadata, but are generated AFTER constraints
-            if num_col in output_df.columns:
-                output_df[num_col] = pd.to_numeric(
-                    output_df[num_col], errors="coerce").astype("float64")
 
         clean_rule = clean_up_constraint(bool_constraint)
         
@@ -220,11 +223,16 @@ class ConstraintHandler:
         (self.dependent_column,
         op,
         self.independent_expression) = tokenise_constraint(clean_rule)
+
+        target_col = self.dependent_column.replace("__", " ")
         
         #add the expression we're adjusting TO to the dataframe so that it is
         #available in all apply calls. Maybe move to its own function
         if self.independent_expression.isdigit():
 
+            output_df["test_expression"] = int(self.independent_expression)
+
+        elif self.independent_expression.replace(",","").replace(".","").isdigit():
             output_df["test_expression"] = float(self.independent_expression)
 
         elif self.is_independent_expression_an_iso_date(self.independent_expression):
@@ -240,25 +248,21 @@ class ConstraintHandler:
 
             output_df["test_expression"] = (output_df
                                 .rename(lambda x: x.replace(" ", "__"), axis="columns")
-                                .eval(self.independent_expression, engine="python"))
+                                .eval(self.independent_expression, engine="python")
+                                )
 
-        output_df.loc[~mask, self.dependent_column.replace("__", " ")] = (
+        output_df.loc[~mask, target_col] = (
             output_df[~mask]
-                .rename(lambda x: x.replace(" ", "__"), axis="columns")
                 .apply(
                     self.adjust_value_to_constraint,
                     axis=1,
-                    args=(
-                        self.dependent_column,
-                        op)
+                    args=(target_col, op)
                 )
         )
 
-        target_col = self.dependent_column.replace("__", " ")
-
         precision = self.spec_dict["columns"][target_col].get("precision", None)
         
-        if precision == 'integer':
+        if precision == "integer":
             output_df[target_col] = output_df[target_col].round().astype("Int64")
 
         #drop the test_expression column
@@ -343,32 +347,35 @@ class ConstraintHandler:
 
         # adjust only if dispersion % is provided in the spec
         adj_factor = y * root["distribution_parameters"].get("dispersion", 0)
+        int_precision = root.get("precision", "integer") == "integer"
 
         # if dispersion is zero, pick the next valid value
         if adj_factor == 0:
 
             if op.__name__ == "less":
-                return max(0, y - 1)
+                return np.floor(max(0, y - 1)) if int_precision else max(0, y - 1)
             if op.__name__ == "greater":
-                return y + 1 #pragma: no cover
+                return np.ceil(y + 1) if int_precision else y + 1 #pragma: no cover
             return y
 
         # new x value is drawn from the dispersion-based interval around y
         if "less" in op.__name__:
             new_x_max = y
             new_x_min = max(0, y - adj_factor)
-            return self.recursive_randint(new_x_min, new_x_max, y, op)
+            result = self._random_value_from_interval(new_x_min, new_x_max, y, op)
+            return np.floor(result) if int_precision else result
         if "greater" in op.__name__:
             new_x_min = y
             new_x_max = y + adj_factor
-            return self.recursive_randint(new_x_min, new_x_max, y, op)
+            result = self._random_value_from_interval(new_x_min, new_x_max, y, op)
+            return np.ceil(result) if int_precision else result
         # only option left is constraint is x == y so we return y
         return y #pragma: no cover
         
 
-    def recursive_randint(self, new_x_min, new_x_max, y, op, n=100):
+    def _random_value_from_interval(self, new_x_min, new_x_max, y, op, n=100):
         '''
-        Helper function to generate a random integer that conforms
+        Helper function to generate a random value that conforms
         to the given constraint.
 
         Occasionally, you might get into a situation when determining
@@ -377,14 +384,14 @@ class ConstraintHandler:
         the constraint operator.
         '''
 
-        new_x = round(self.rng.uniform(new_x_min, new_x_max))
+        new_x = self.rng.uniform(new_x_min, new_x_max)
         n = n - 1
 
-        if n > 0:        
+        if n > 0:
 
             if op(new_x, y):
                 return new_x
-            return self.recursive_randint(new_x_min, new_x_max, y, op, n)
+            return self._random_value_from_interval(new_x_min, new_x_max, y, op, n)
     
         if op.__name__ == "less":
             return max(0, y - 1)
@@ -394,11 +401,17 @@ class ConstraintHandler:
 
     # CONDITIONAL CONSTRAINT FUNCTIONS
     # ================================
-    # Every custom function MUST implement the same call signature
+    # Every custom function MUST implement the same call signature and return a DF
     # If adding a new function, don't forget to add it to the dispatch dict as well
     # Each function should also handle missing data and partitioning logic
+
+    # since process_constraints doesn't know about what the action function wants to
+    # do with the target_cols - whether columns will be operated at together or in 
+    # turn, whether one column will be returned or multiple (like in the case of geo
+    # spatial actions returning lat / long columns - so it expects all columns at
+    # filter_idx to be returned.
     def make_outlier(
-        self, df, filter_idx, target_col, partition_cols=None, rescale=True):
+        self, df, filter_idx, target_str, partition_cols=None, rescale=True):
         '''
         Make filtered data slice an outlier compared to the rest of the data
         included in the partition (or entire dataset) using boxplot methodology
@@ -409,8 +422,8 @@ class ConstraintHandler:
             Unmodified dataframe
         filter_idx     : pd.Index
             Index of rows to be modified by the function
-        target_col     : str
-            Column where user wants to add outliers
+        target_str     : str
+            Column(s) where user wants to add outliers
         partition_cols : list
             Columns to group by before computing IQR ranges and outliers
         rescale        : boolean
@@ -419,8 +432,8 @@ class ConstraintHandler:
 
         Returns
         -------
-        pd.Series
-            New series with outliers substituted for masked values
+        pd.DataFrame
+            New DF with 1 or more columns with outliers substituted for masked values
         '''
 
         def _within_group_outliers(series):
@@ -452,52 +465,67 @@ class ConstraintHandler:
 
             return outlier_series
 
-        series = df[target_col].dropna()
-        # make sure that original filtered index reflects the non-null series
-        filter_idx = series.index.intersection(filter_idx)
-
+        final_result = []
+        target_cols = [x.strip() for x in target_str.split(",")]
         if partition_cols is not None:
-
             partition_cols = [x.strip() for x in partition_cols.split(",") if x]
-            grouped_series = df.dropna(subset=[target_col]).groupby(partition_cols)[target_col]
-            outlier_series = grouped_series.transform(_within_group_outliers)
-            result = series.copy()
-            result.loc[filter_idx] = outlier_series.loc[filter_idx]
 
-        else:
+        for target_col in target_cols:
+        
+            series = df[target_col].dropna()
+            # make sure that original filtered index reflects the non-null series
+            filter_idx = series.index.intersection(filter_idx)
 
-            q25, q50, q75 = np.percentile(series, [25, 50, 75])
-            iqr = q75 - q25
+            if partition_cols is not None:
 
-            if iqr == 0:
-
-                masked_series = np.where(
-                    series.loc[filter_idx] % 2 == 0,
-                    series.loc[filter_idx] * 1.3,
-                    series.loc[filter_idx] * 0.7
-                )
+                grouped_series = df.dropna(subset=[target_col]).groupby(partition_cols)[target_col]
+                outlier_series = grouped_series.transform(_within_group_outliers)
+                result = series.copy()
+                result.loc[filter_idx] = outlier_series.loc[filter_idx]
 
             else:
-                # make outliers in the same direction as original values
-                masked_series = np.where(
-                    series.loc[filter_idx] >= q50,
-                    (q75 + iqr * 3) + series.loc[filter_idx],
-                    (q25 - iqr * 3) - series.loc[filter_idx]
-                )
 
-            result = series.copy()
-            result.loc[filter_idx] = masked_series
+                q25, q50, q75 = np.percentile(series, [25, 50, 75])
+                iqr = q75 - q25
 
-        if rescale:
-            col_data = self.spec_dict["columns"][series.name]
-            precision = col_data["precision"]
-            dist_params = col_data["distribution_parameters"]
-            result = scale_continuous_column(result, precision, **dist_params)
+                if iqr == 0:
 
-        return result
+                    masked_series = np.where(
+                        series.loc[filter_idx] % 2 == 0,
+                        series.loc[filter_idx] * 1.3,
+                        series.loc[filter_idx] * 0.7
+                    )
+
+                else:
+                    # make outliers in the same direction as original values
+                    masked_series = np.where(
+                        series.loc[filter_idx] >= q50,
+                        (q75 + iqr * 3) + series.loc[filter_idx],
+                        (q25 - iqr * 3) - series.loc[filter_idx]
+                    )
+
+                result = series.copy()
+                result.loc[filter_idx] = masked_series
+
+            if rescale:
+                col_data = self.spec_dict["columns"][series.name]
+                precision = col_data["precision"]
+                dist_params = col_data["distribution_parameters"]
+                result = scale_continuous_column(result, precision, **dist_params)
+
+            final_result.append(result)
+
+        # return a new dataframe with amended target_cols, plus the unchanged rest
+        new_df = pd.concat(
+            final_result + 
+            [df.loc[filter_idx, [x for x in df.columns if x not in target_cols]]],
+            axis=1
+        )
+
+        return new_df
 
     def sort_values(
-        self, df, filter_idx, target_col, partition_cols=None, ascending=True):
+        self, df, filter_idx, target_str, partition_cols=None, asc=True):
         '''
         Sort filtered data slice with optional nesting achieved by partitioning
 
@@ -507,39 +535,55 @@ class ConstraintHandler:
             Unmodified dataframe
         filter_idx     : pd.Index
             Index of rows to be modified by the function
-        target_col     : str
-            Column where user wants to add outliers
+        target_str     : str
+            Column(s) where user wants to add outliers
         partition_cols : list
             Columns to group by to achieve nested sort
-        ascending        : boolean
+        asc            : boolean
             Direction of sorting
 
         Returns
         -------
-        pd.Series
-            Only the filtered and sorted data slice is returned, not the whole series
+        pd.DataFrame
+            Only the filtered and sorted slice is returned
         '''
 
-        if partition_cols is None:
-            
-            new_sorted_series = df.loc[filter_idx, target_col].sort_values(ascending=ascending)
-            new_sorted_series.index = filter_idx
-            return new_sorted_series
+        final_result = []
+        target_cols = [x.strip() for x in target_str.split(",")]
 
-        partition_cols = [x.strip() for x in partition_cols.split(",") if x]
+        if partition_cols is not None:
+            partition_cols = [x.strip() for x in partition_cols.split(",") if x]
 
-        # remember that sorted() defaults to reverse=False so
-        # sorted(False) = ascending; df.sort_values(False) = descending
-        # meaning for sorted() we have to reverse the boolean parameter
-        result = (df
-            .groupby(partition_cols)[target_col]
-            .transform(sorted, reverse=not ascending)
-            .loc[filter_idx]
+        for target_col in target_cols:
+
+            if partition_cols is None:
+                
+                new_series = df.loc[filter_idx, target_col].sort_values(ascending=asc)
+                new_series.index = filter_idx
+                final_result.append(new_series)
+                continue
+
+            # remember that sorted() defaults to reverse=False so
+            # sorted(False) = ascending; df.sort_values(False) = descending
+            # meaning for sorted() we have to reverse the boolean parameter
+            result = (df
+                .groupby(partition_cols)[target_col]
+                .transform(sorted, reverse=not asc)
+                .loc[filter_idx]
+            )
+
+            final_result.append(result)
+
+        # return a new dataframe with amended target_cols, plus the unchanged rest
+        new_df = pd.concat(
+            final_result + 
+            [df.loc[filter_idx, [x for x in df.columns if x not in target_cols]]],
+            axis=1
         )
 
-        return result
+        return new_df
 
-    def make_distinct(self, df, filter_idx, target_col, partition_cols=None):
+    def make_distinct(self, df, filter_idx, target_str, partition_cols=None):
         '''
         Ensure the filtered data slice has distinct values within the specified
         partition. Paired columns are not supported yet.
@@ -550,8 +594,8 @@ class ConstraintHandler:
             Unmodified dataframe
         filter_idx     : pd.Index
             Index of rows to be modified by the function
-        target_col     : str
-            Column where user wants to add outliers
+        target_str     : str
+            Column(s) where user wants values to be distinct
         partition_cols : list
             Columns to group by before computing IQR ranges and outliers
 
@@ -575,47 +619,65 @@ class ConstraintHandler:
 
             new_group = group.where(~group.duplicated(), "").tolist()
             
-            return new_group
+            return pd.Series(new_group, name=target_col)
 
-        orig_vals_in_spec = self.spec_dict["columns"][target_col]["original_values"]
+        final_result = []
+        target_cols = [x.strip() for x in target_str.split(",")]
 
-        if isinstance(orig_vals_in_spec, pd.DataFrame):
-            original_uniques = (
-                orig_vals_in_spec[target_col].tolist()
+        if partition_cols is not None:
+            partition_cols = [x.strip() for x in partition_cols.split(",") if x]
+
+        # the section on orig_vals / orig_uniques is not needed
+        for target_col in target_cols:
+
+            orig_vals_in_spec = self.spec_dict["columns"][target_col]["original_values"]
+
+            if isinstance(orig_vals_in_spec, pd.DataFrame):
+                original_uniques = (
+                    orig_vals_in_spec[target_col].tolist()
+                )
+
+            elif orig_vals_in_spec == ORIGINAL_VALUES_DB:
+
+                table_name = f'temp_{self.spec_dict["metadata"]["id"]}_{target_col}'
+                original_uniques = query_anon_database(
+                    table_name=table_name,
+                    column=target_col,
+                )[target_col].tolist()
+
+            elif orig_vals_in_spec == ORIGINAL_VALUES_PAIRED: #pragma: no cover
+                raise ValueError(
+                    f"{self.current_action} not supported for paired columns")
+
+            if MISSING_DATA_STR in original_uniques:
+                original_uniques.remove(MISSING_DATA_STR)
+                    
+            if partition_cols is None:
+                
+                filtered_series = df.loc[filter_idx, target_col]
+                result = _make_distinct_within_group(filtered_series)
+                final_result.append(result)
+                continue
+
+            result = (df
+                .loc[filter_idx]
+                .groupby(partition_cols)[target_col]
+                .transform(_make_distinct_within_group)
             )
 
-        elif orig_vals_in_spec == ORIGINAL_VALUES_DB:
+            final_result.append(result)
 
-            table_name = f'temp_{self.spec_dict["metadata"]["id"]}_{target_col}'
-            original_uniques = query_anon_database(
-                table_name=table_name,
-                column=target_col,
-            )[target_col].tolist()
-
-        elif orig_vals_in_spec == ORIGINAL_VALUES_PAIRED: #pragma: no cover
-            raise ValueError("make_distinct action not supported for paired columns")
-
-        if MISSING_DATA_STR in original_uniques:
-            original_uniques.remove(MISSING_DATA_STR)
-                 
-        if partition_cols is None:
-            
-            filtered_series = df.loc[filter_idx, target_col]
-            result = _make_distinct_within_group(filtered_series)
-            return result
-
-        partition_cols = [x.strip() for x in partition_cols.split(",") if x]
-
-        result = (df
-            .loc[filter_idx]
-            .groupby(partition_cols)[target_col]
-            .transform(_make_distinct_within_group)
+        # return a new dataframe with amended target_cols, plus the unchanged rest
+        new_df = pd.concat(
+            final_result + 
+            [df.loc[filter_idx, [x for x in df.columns if x not in target_cols]]],
+            axis=1
         )
 
-        return result
+        return new_df
 
     def make_same(
-        self, df, filter_idx, target_col, partition_cols=None):
+        self, df, filter_idx, target_str, partition_cols=None):
         '''
         Force all values in the partition to be the same as the first.
         Remember that groubpy doesn't sort the observations within groups
@@ -628,40 +690,54 @@ class ConstraintHandler:
             Unmodified dataframe
         filter_idx     : pd.Index
             Index of rows to be modified by the function
-        target_col     : str
-            Column where user wants to add outliers
+        target_str     : str
+            Column(s) where user wants to make same
         partition_cols : list
             Columns to group by to achieve nested sort
 
         Returns
         -------
-        pd.Series
-            Only the filtered data slice with identicl values is returned,
-            not the whole series
+        pd.DataFrame
+            Only the filtered data slice with identical values is returned
         '''
+       
+        final_result = []
+        target_cols = [x.strip() for x in target_str.split(",")]
+        if partition_cols is not None:
+            partition_cols = [x.strip() for x in partition_cols.split(",") if x]
 
-        if partition_cols is None:
-            
-            repl = df.loc[filter_idx[0], target_col]
-            new_same_series = df.loc[filter_idx, target_col].transform(lambda x: repl)
-            return new_same_series
+        for target_col in target_cols:
 
-        partition_cols = [x.strip() for x in partition_cols.split(",") if x]
+            if partition_cols is None:
+                
+                repl = df.loc[filter_idx[0], target_col]
+                new_series = df.loc[filter_idx, target_col].transform(lambda x: repl)
+                final_result.append(new_series)
+                continue
 
-        result = (df
-            .groupby(partition_cols)[target_col]
-            .transform(lambda x: x.iloc[0])
-            .loc[filter_idx]
+            temp_result = (df
+                .groupby(partition_cols)[target_col]
+                .transform(lambda x: x.iloc[0])
+                .loc[filter_idx]
+            )
+
+            final_result.append(temp_result)
+
+        # return a new dataframe with amended target_cols, plus the unchanged rest
+        new_df = pd.concat(
+            final_result + 
+            [df.loc[filter_idx, [x for x in df.columns if x not in target_cols]]],
+            axis=1
         )
 
-        return result
+        return new_df
 
     def generate_as_sequence(
-        self, df, filter_idx, target_col, partition_cols=None):
+        self, df, filter_idx, target_str, partition_cols=None, repeating=False):
         '''
         This custom constraint is only valid if the original values of the target
         column are below the in-line limit and appear in the spec. Taking them from
-        the DB won't work because the order is not guaranteed.
+        the DB won't work because the order is not guaranteed, except if you sort.
 
         Because the first value in the spec order is guaranteed to be included, you
         should compensate for it in the probabilities of other values. For example,
@@ -680,10 +756,15 @@ class ConstraintHandler:
             Unmodified dataframe
         filter_idx     : pd.Index
             Index of rows to be modified by the function
-        target_col     : str
-            Column where user wants to add outliers
+        target_str     : str
+            Column where user wants values to follow the spec order
         partition_cols : list
             Columns to group by
+        repeating: boolean
+            If set to True, when all values in a sequence have equal probabilities
+            and the number of values in a partition exceeds the number of requested
+            rows, the sequence will restart. This is different from normal mode where
+            the combined probabilities dictate whether a sequence will reach its end. 
 
         Returns
         -------
@@ -691,15 +772,6 @@ class ConstraintHandler:
             Only the filtered data slice is returned not the whole series
         '''
 
-        orig_vals = self.spec_dict["columns"][target_col]["original_values"]
-
-        if not isinstance(orig_vals, pd.DataFrame): #pragma: no cover
-            print("WARNING: Values are missing from the spec.")
-            return df.loc[filter_idx, target_col]
-
-        ordered_list = orig_vals[target_col].tolist()
-        ordered_probs = orig_vals["probability_vector"].tolist()
-
         def _generate_ordered_values(target_sequence, ordered_list, ordered_probs):
             '''
             Helper function to deal with padding; returns a list
@@ -715,12 +787,24 @@ class ConstraintHandler:
             m = len(ordered_list)
             unordered_result = []
             pointer = 0
+
+            if repeating:
+                full_seq_i = int(np.floor(n / m))
+                remainder_seq_j = int(n % m)
+
+                result = (
+                    ordered_list * full_seq_i +
+                    ordered_list[:remainder_seq_j]
+                )
+
+                return result
             
             # special case; ignore probabilities
             if len(set(ordered_probs)) == 1:
                 
                 # more rows than available values
                 if (diff := n - m) > 0:
+
                     return ordered_list + [""] * diff
 
                 return ordered_list[:n]
@@ -744,26 +828,223 @@ class ConstraintHandler:
 
             result = sorted(unordered_result, key=lambda x: ordered_list.index(x))
 
-            return result            
+            return result  
+
+        final_result = []
+        target_cols = [x.strip() for x in target_str.split(",")]
+        if partition_cols is not None:
+            partition_cols = [x.strip() for x in partition_cols.split(",") if x]
+
+        for target_col in target_cols:
+
+            orig_vals = self.spec_dict["columns"][target_col]["original_values"]
+
+            if not isinstance(orig_vals, pd.DataFrame): #pragma: no cover
+                print("WARNING: Values are missing from the spec.")
+                final_result.append(df.loc[filter_idx, target_col])
+                continue
+
+            ordered_list = orig_vals[target_col].tolist()
+            ordered_probs = orig_vals["probability_vector"].tolist()
+
+            if partition_cols is None:
+                
+                new_vals = _generate_ordered_values(
+                    filter_idx, ordered_list, ordered_probs)
+                final_result.append(pd.Series(new_vals, index=filter_idx))
+                continue
+
+            result = (df
+                .groupby(partition_cols)[target_col]
+                .transform(
+                    _generate_ordered_values,
+                    ordered_list=ordered_list,
+                    ordered_probs=ordered_probs,
+                )
+                .loc[filter_idx]
+            )
+
+            final_result.append(result)
+
+        # return a new dataframe with amended target_cols, plus the unchanged rest
+        new_df = pd.concat(
+            final_result + 
+            [df.loc[filter_idx, [x for x in df.columns if x not in target_cols]]],
+            axis=1
+        )
+
+        return new_df
+
+    def sort_and_skew(
+        self, df, filter_idx, target_str, partition_cols=None, direction=None):
+        '''
+        Transform two columns to create a custom distribution.
+        If both columns and continuous, the first one in target_str is sorted and the
+        second is transformed to have its distribution skewed.
+
+        Parameters
+        ----------
+        df             : pd.DataFrame
+            Unmodified dataframe
+        filter_idx     : pd.Index
+            Index of rows to be modified by the function
+        target_str     : str
+            target_str must have at least two columns
+        partition_cols : list
+            Columns to group by if you want to retain the scaling (to a target sum) per
+            group.
+        direction      : str
+            Direction of the new distribution based on a sine function. One of: left,
+            right, up, down.
+
+        Returns
+        -------
+        pd.DataFrame
+            Only the filtered slice is returned
+        '''
+        # dictionary with parameters for the sine function depending on the type of 
+        # distribution the user requests
+        # the tuple is for:
+        # a - amplitude, b - angular velocity (width), c - phase angle (shift)
+        sine_dict = {
+            "right": (1, 5, 0.9),
+            "left" : (-1, 5, 0.4),
+            "up"   : (-1, 7.6, 0.9),
+            "down" : (1, 7.6, 0.9),
+        }
+
+        def _make_skewed_series(group):
+            '''
+            Logic for transformation. Although we rescale the skewed series, if the
+            specification doesn't allow duplicates, the final sum might differ from
+            the expected. Also, remember that by skewing the values from the original
+            distribution, the row weights are destroyed via sorting - you can't have
+            a specific looking distribution and weights that work against it. There
+            could be four directions of skew: left (mean to the left of the median),
+            right (mean to the right of the median), peak and valley. All of these
+            are achieved by using sine waves with different parameters. 
+            '''
+
+            nrows = len(group)
+            if nrows == 1: #pragma: no cover
+                return group
+
+            sine_params = sine_dict.get(direction, None)
+            if sine_params is None: #pragma: no cover
+                return group
+
+            x = np.linspace(0, 1, num=nrows)
+            a, b, c = sine_params
+
+            sine_vals = a * np.sin(b * x + c)
+            sine_vals = sine_vals + rng.uniform(low=-0.5, high=0.5, size=nrows)
+            skewed_series = pd.Series(data=sine_vals, name=skew_col)
+        
+            scale_params = {}
+
+            # remember that when creating a peak, you want to continue from previous
+            # point so the distribution needs to be "lifted" up by half. Same with the
+            # valley. Skew left / right have slight coefficients to make the curve look
+            # better. 
+            scale_coefs = {
+                "sum" : {
+                    "up"   : 1.5,
+                    "down" : 0.5,
+                    "left" : 0.8,
+                    "right": 0.8,
+                },
+                "min" : {
+                    "up"   : 1,
+                    "down" : 0.5,
+                    "left" : 0.5,
+                    "right": 0.5,
+                },
+                "max" : {
+                    "up"   : 1.5,
+                    "down" : 1,
+                    "left" : 1,
+                    "right": 1,
+                },
+            }
+
+            if dist_params_set.intersection(set(["target_sum"])):
+                scale_params["target_sum"] = group.sum() * scale_coefs["sum"][direction]
+                        
+            elif dist_params_set.intersection(set(["target_min", "target_max"])):
+                scale_params["target_min"] = group.min() * scale_coefs["min"][direction]
+                scale_params["target_max"] = group.max() * scale_coefs["max"][direction]
+
+            elif dist_params_set.intersection(set(["target_mean", "target_std"])):
+                scale_params["target_mean"] = group.mean()
+                scale_params["target_std"] = group.std() * 2
+
+            if scale_params:
+                result = scale_continuous_column(
+                    skewed_series, precision, **scale_params)
+            else: #pragma: no cover
+                result = skewed_series
+
+            # add nulls based on the miss_probability of the skew column
+            miss_pct = self.spec_dict["columns"][skew_col]["miss_probability"]
+
+            skewed_result = np.where(
+                rng.random(size=nrows) < miss_pct,
+                pd.NA, result.values)
+
+            skewed_series = pd.Series(
+                data=skewed_result, index=group.index, name=skew_col)
+            
+            return skewed_series
+
+        target_cols = [x.strip() for x in target_str.split(",")]
+        if len(target_cols) != 2: # pragma: no cover
+            raise Exception(f"{self.current_action} requires exactly 2 target columns.")
+
+        if partition_cols is not None:
+            partition_cols = [x.strip() for x in partition_cols.split(",") if x]
+        
+        sort_col = target_cols[0]
+        skew_col = target_cols[1]
+        
+        rng = self.spec_dict["_rng"]
+        col_data = self.spec_dict["columns"][skew_col]
+        precision = col_data.get("precision", "integer")
+        dist_params_set = set(col_data["distribution_parameters"].keys())
 
         if partition_cols is None:
             
-            new_vals = _generate_ordered_values(filter_idx, ordered_list, ordered_probs)
-            return pd.Series(new_vals, index=filter_idx)
+            series = df.loc[filter_idx, skew_col]
+            sorted_series = df.loc[filter_idx, sort_col].sort_values()
+            skewed_series = _make_skewed_series(series)
+            sorted_series.index = skewed_series.index
 
-        partition_cols = [x.strip() for x in partition_cols.split(",") if x]
-
-        result = (df
-            .groupby(partition_cols)[target_col]
-            .transform(
-                _generate_ordered_values,
-                ordered_list=ordered_list,
-                ordered_probs=ordered_probs,
+            new_df = pd.concat(
+                [skewed_series, sorted_series] + 
+                [df.loc[filter_idx, [x for x in df.columns if x not in target_cols]]],
+                axis=1
             )
+
+            return new_df
+
+        skewed_series = (df
             .loc[filter_idx]
+            .groupby(partition_cols)[skew_col]
+            .transform(_make_skewed_series)
         )
 
-        return result
+        sorted_series = (df
+                .loc[filter_idx]
+                .groupby(partition_cols)[sort_col]
+                .transform(sorted)  
+            )
+
+        new_df = pd.concat(
+            [skewed_series, sorted_series] + 
+            [df.loc[filter_idx, [x for x in df.columns if x not in target_cols]]],
+            axis=1
+        )
+
+        return new_df        
 
 # EXPORTABLE METHODS
 # ==================
