@@ -201,6 +201,16 @@ class newExhibit:
         else: #pragma: no cover
             raise TypeError("Specification is not in .yml format")
 
+        # for convenience, some YAML heading, like numerical_columns or linked_columns
+        # can be left blank by the user; this is read as NONE by YAML parser, but the
+        # code expects them to be sequences (of column names) - as a bandaid, we catch
+        # these NONE values early and change them into empty sequences.
+        for key, value in self.spec_dict["metadata"].items():
+            if "columns" in key and value is None:
+                self.spec_dict["metadata"][key] = set()
+
+        if self.spec_dict.get("linked_columns", None) is None:
+                self.spec_dict["linked_columns"] = list()
 
         for col in self.spec_dict["metadata"]["categorical_columns"]:
 
@@ -296,9 +306,11 @@ class newExhibit:
             if custom_constraints:
                 for _, cc in custom_constraints.items():
                     cc_targets = cc["targets"]
-                    for target_col, target_action in cc_targets.items():
+                    for target_str, target_action in cc_targets.items():
                         if target_action[:3] == "geo":
-                            geo_action_targets.append(target_col)
+                            geo_action_targets.extend(
+                                [x.strip() for x in target_str.split(",")]
+                            )
             
             if col in geo_action_targets:
                 # add placeholders to avoid errors when generating missing data
@@ -349,12 +361,18 @@ class newExhibit:
         if (ccs:=self.spec_dict["constraints"]["custom_constraints"]):
             
             for _, cc in ccs.items():
-                constraint_targets.extend(cc["targets"].keys())
+                for targets_string in cc["targets"].keys():
+                    constraint_targets.extend(
+                        [x.strip() for x in targets_string.split(",")])
 
         # check if there are common columns between constraint targets and cat_cols
         if cat_cols_set & set(constraint_targets):
             
-            num_cols = self.spec_dict["metadata"]["numerical_columns"]
+            # we don't want to re-create continuous columns that were subject of a cc
+            num_cols = (
+                set(self.spec_dict["metadata"]["numerical_columns"]) -
+                set(constraint_targets)
+            )
             for num_col in num_cols:
                 
                 # derived columns won't have weight so we ignore them
