@@ -99,7 +99,7 @@ def scale_continuous_column(series, precision, **dist_params):
     # fallback is to return unscaled series
     return series
 
-def generate_derived_column(anon_df, calculation, precision=2):
+def generate_derived_column(anon_df, calculation):
     '''
     Use Pandas eval() function to try to parse user calculations.
 
@@ -110,8 +110,6 @@ def generate_derived_column(anon_df, calculation, precision=2):
         dataframe is nearly complete
     calculation : str
         user-defined calculation to create a new column
-    precision : int
-        output is rounded to the given precision
 
     Returns
     --------
@@ -130,6 +128,8 @@ def generate_derived_column(anon_df, calculation, precision=2):
 
     # some columns might come in as Int64 (Pandas dtype) which will fall over
     # when trying to use in eval because it relies on numpy dtypes, not pandas'
+    # this means that it's safer to let the dtype fall down to float even though
+    # it might be the result of running an operation between two INT columns.
     numeric_cols = safe_df.select_dtypes(include=np.number).columns
     safe_df[numeric_cols] = safe_df[numeric_cols].astype(float)
 
@@ -137,7 +137,6 @@ def generate_derived_column(anon_df, calculation, precision=2):
         #groupby requires pd.eval, not df.eval
         temp_series = (pd
                     .eval(safe_calculation, local_dict={"df":safe_df}, engine="python")
-                    .round(precision)
                 )
 
         #expect multi-index in groupby DF
@@ -153,10 +152,9 @@ def generate_derived_column(anon_df, calculation, precision=2):
 
     basic_output = (safe_df
                     .eval(safe_calculation, local_dict={"df":safe_df}, engine="python")
-                    .round(precision)
                 )
 
-    return basic_output  
+    return basic_output
 
 def generate_cont_val(
     row,
@@ -327,7 +325,7 @@ def _scale_to_target_sum(series, precision, target_sum, **_kwargs):
     if precision == "integer":
 
         rounded_scaled_series = _conditional_rounding(scaled_series, target_sum)
-        return rounded_scaled_series
+        return rounded_scaled_series.astype("Int64")
     
     return round(scaled_series, 4)
 
