@@ -334,12 +334,13 @@ class linkageTests(unittest.TestCase):
         This happens when the number of unique values in each column
         exceeds the user-specified threshold. In this case, the values
         are stored in anon.db and the user has no way to specify bespoke
-        probabilities.
+        probabilities. All SQL DB linked tables will have Missing Data as
+        the last row.
         '''
 
         sql_df = pd.DataFrame(data={
-            "A":sorted([f"A{i}" for i in range(5)]*2),
-            "B": [f"B{i}" for i in range(10)]
+            "A":list(sorted([f"A{i}" for i in range(5)]*2)) + ["Missing data"],
+            "B": [f"B{i}" for i in range(10)] + ["Missing data"]
         })
 
         #we're bypassing __init__ and going straight to testing scenario code
@@ -464,7 +465,7 @@ class linkageTests(unittest.TestCase):
                     "original_values":
                         pd.DataFrame(data={
                             "B": [f"B{i}" for i in range(4)] + [MISSING_DATA_STR],
-                            "probability_vector": [0.1, 0.1, 0.4, 0.4, 0]
+                            "probability_vector": [0.101, 0.101, 0.4, 0.4, 0]
                         })
                 },
                 "C": {
