@@ -32,7 +32,7 @@ class constraintsTests(unittest.TestCase):
             anon_df=pd.DataFrame(),
         )
 
-    def test_recursive_randint_error_handling(self):
+    def test_random_value_from_interval_error_handling(self):
         '''
         Return a matching value given left side, right side and operator
         '''
@@ -46,7 +46,7 @@ class constraintsTests(unittest.TestCase):
         for op, val in zip(ops, target_vals):
         
             result.append(
-                self.ch.recursive_randint(0, 10000, val, op)
+                self.ch._random_value_from_interval(0, 10000, val, op)
             )
 
         self.assertCountEqual(result, expected)
@@ -170,7 +170,9 @@ class constraintsTests(unittest.TestCase):
     def test_adjust_value_to_constraint_scalar_uniform(self):
         '''
         The column that is being adjusted to a scalar is generated
-        using weighted uniform parameters (dispersion)
+        using weighted uniform parameters (dispersion). Note that with
+        dispersion set to zero the floating number column will simply have
+        target - 1 value.
         '''
 
         self.ch.spec_dict["metadata"] = {"numerical_columns": ["A A", "B"]}
@@ -184,6 +186,7 @@ class constraintsTests(unittest.TestCase):
             },
             "B": {
                 "type": "continuous",
+                "precision" : "float",
                 "distribution": "weighted_uniform",
                 "distribution_parameters": {
                     "dispersion": 0
@@ -191,23 +194,24 @@ class constraintsTests(unittest.TestCase):
             },
             }
 
-        self.ch.dependent_column = "A"
-
         test_df = pd.DataFrame(
             data={
-                "A A":[pd.NA, 0, 20, 2, 50],
-                "B":[1, 5, 21, 1, 1000]
+                "A A":[np.nan, 0, 20, 2, 50],
+                "B":[0.5, 1.6, 1.2, 1.5, 0.1]
             }
         )
 
-        constraint_gr = "~A A~ > 30"
-        constraint_ls = "~A A~ < 30"
+        int_gr = "~A A~ > 30"
+        int_ls = "~A A~ < 30"
+        float_ls = "B < 1.5"
 
-        result_gr = self.ch.adjust_dataframe_to_fit_constraint(test_df, constraint_gr)
-        result_ls = self.ch.adjust_dataframe_to_fit_constraint(test_df, constraint_ls)
+        result_int_gr = self.ch.adjust_dataframe_to_fit_constraint(test_df, int_gr)
+        result_int_ls = self.ch.adjust_dataframe_to_fit_constraint(test_df, int_ls)
+        result_float_ls = self.ch.adjust_dataframe_to_fit_constraint(test_df, float_ls)
 
-        self.assertTrue(all(result_gr["A A"].dropna() > 30))
-        self.assertTrue(all(result_ls["A A"].dropna() < 30))
+        self.assertTrue(all(result_int_gr["A A"].dropna() > 30))
+        self.assertTrue(all(result_int_ls["A A"].dropna() < 30))
+        self.assertTrue(all(result_float_ls["B"].dropna() < 1.5))
 
     def test_adjust_value_to_constraint_scalar_normal(self):
         '''
@@ -837,17 +841,6 @@ class constraintsTests(unittest.TestCase):
         test_dict = {
 
             "_rng" : np.random.default_rng(seed=0),
-            "columns" : {
-                "A" : {
-                    "uniques" : 5,
-                    "original_values" : pd.DataFrame(data={"A": list("ABCDE")})
-                },
-                "C" : {
-                    "uniques" : 20,
-                    "original_values": pd.DataFrame(
-                        data={"C": [str(x) for x in range(20)]})
-                }
-            },
             "constraints" : {
                 "custom_constraints": {
                     "cc1" : {
@@ -879,12 +872,6 @@ class constraintsTests(unittest.TestCase):
         test_dict = {
 
             "_rng" : np.random.default_rng(seed=0),
-            "columns" : {
-                "A" : {
-                    "uniques" : 5,
-                    "original_values" : pd.DataFrame(data={"A": list("ABCDE")})
-                },
-            },
             "constraints" : {
                 "custom_constraints": {
                     "cc1" : {
@@ -1033,6 +1020,296 @@ class constraintsTests(unittest.TestCase):
 
         self.assertListEqual(expected, result)
 
+    def test_custom_constraints_generate_as_repeating_sequence(self):
+        '''
+        Doc string
+        '''
+
+        test_dict = {
+
+            "_rng" : np.random.default_rng(seed=0),
+            "columns" : {
+                "A" : {
+                    "uniques" : 3,
+                    "original_values" : pd.DataFrame(data={
+                        "A": list("ABC"),
+                        "probability_vector": [0.25, 0.25, 0.25]
+                        })
+                },
+            },
+            "constraints" : {
+                "custom_constraints": {
+                    "cc1" : {
+                        "partition" : "B",
+                        "targets" : {
+                            "A" : "generate_as_repeating_sequence",
+                        }
+                    }
+                }
+            },
+        }
+
+        test_data = pd.DataFrame(data={
+            "A" : list("ABBAC") * 2,
+            "B" : ["p1"] * 8 + ["p2"] * 2
+        })
+
+        test_gen = tm.ConstraintHandler(test_dict, test_data)
+        anon_df = test_gen.process_constraints()
+        
+        expected = ["A","B","C","A","B","C","A","B","A","B"]
+        result = anon_df["A"].tolist()
+
+        self.assertListEqual(expected, result)
+
+    def test_custom_constraints_one_targets_multiple_actions(self):
+        '''
+        Users should be able to apply multiple actions (comma separated) to the target
+        columns. Remember that under the latest implementation, make_distinct won't add
+        any new values to the partition - just remove duplicates and replace them with
+        blank "" strings to preserve distributions better.
+        '''
+
+        test_dict = {
+
+            "_rng" : np.random.default_rng(seed=0),
+            "columns" : {
+                "A" : {
+                    "uniques" : 4,
+                    "original_values" : pd.DataFrame(data={
+                        "A": list("ABCD"),
+                        "probability_vector": [0.1, 0.1, 0.4, 0.4]
+                        })
+                },
+            },
+            "constraints" : {
+                "custom_constraints": {
+                    "cc1" : {
+                        "targets" : {
+                            "A" : "make_distinct, sort_descending",
+                        }
+                    },
+                }
+            },
+        }
+
+        test_data = pd.DataFrame(data={
+            "A" : list("CCDD")
+        })
+
+        test_gen = tm.ConstraintHandler(test_dict, test_data)
+        result = test_gen.process_constraints()["A"].to_list()
+
+        expected = ["D", "C", "", ""]
+        
+        self.assertListEqual(result, expected)
+
+    def test_custom_constraints_multiple_targets_same_action(self):
+        '''
+        Users should be able to apply the same action to multiple targets (comma
+        separated).
+        '''
+
+        test_dict = {
+
+            "_rng" : np.random.default_rng(seed=0),
+            "constraints" : {
+                "custom_constraints": {
+                    "cc1" : {
+                        "filter"  : "C == 'spam'",
+                        "targets" : {
+                            "A, B" : "make_same",
+                        }
+                    },
+                }
+            },
+        }
+
+        test_data = pd.DataFrame(data={
+            "A" : list("ABCDE") * 4,
+            "B" : list("FGHIJ") * 4,
+            "C" : ["spam", "spam", "ham", "ham", "ham"] * 4,
+        })
+
+        test_gen = tm.ConstraintHandler(test_dict, test_data)
+        result = test_gen.process_constraints()
+        
+        self.assertTrue((result.query("C=='spam'")["A"] == "A").all())
+        self.assertTrue((result.query("C=='spam'")["B"] == "F").all())
+
+
+    def test_custom_constraints_sort_and_skew_right(self):
+        '''
+        Mean is to the right of the median. Making a column skewed requires at least
+        two target columns where one is the ordering column and another - whose 
+        distribution is being skewed.
+        '''
+
+        test_dict = {
+
+            "_rng" : np.random.default_rng(seed=0),
+            "columns" : {
+                "A" : {
+                    "type": "continuous",
+                    "precision" : "float",
+                    "distribution" : "weighted_uniform",
+                    "distribution_parameters": {
+                        "dispersion" : 0.1,
+                        "target_sum" : 10_000
+                    },
+                    "miss_probability" : 0,
+                },
+            },
+            "constraints" : {
+                "custom_constraints": {
+                    "cc1" : {
+                        "filter"  : "B >= '2020/03/01'",
+                        "targets" : {
+                            "B, A" : "sort_and_skew_right",
+                        }
+                    },
+                }
+            },
+        }
+
+        test_data = pd.DataFrame(data={
+            "A" : test_dict["_rng"].normal(size=1000),
+            "B": pd.date_range(
+                    start="2020/01/01",
+                    periods=1000,
+                    freq="D",            
+                ),
+        })
+
+        # make sure A is all positive
+        test_data["A"] = test_data["A"] + abs(test_data["A"].min())
+        test_data["B"] = test_data["B"].sample(frac=1)
+
+        test_gen = tm.ConstraintHandler(test_dict, test_data)
+        result = test_gen.process_constraints()
+
+        # make sure that the average of the first 20% of the filtered slice is at least
+        # twice as big as the average of the last 20% of the filtered slice.
+        self.assertGreater(
+            result.query("B >= '2020/03/01'")["A"].iloc[:188].mean(),
+            result.query("B >= '2020/03/01'")["A"].iloc[-188:].mean() * 2
+        )
+ 
+
+    def test_custom_constraints_sort_and_skew_left(self):
+        '''
+        Mean is to the left of the median. Making a column skewed requires at least
+        two target columns where one is the ordering column and another - whose 
+        distribution is being skewed.
+        '''
+
+        test_dict = {
+
+            "_rng" : np.random.default_rng(seed=0),
+            "columns" : {
+                "A" : {
+                    "type": "continuous",
+                    "precision" : "float",
+                    "distribution" : "weighted_uniform",
+                    "distribution_parameters": {
+                        "dispersion" : 0.1,
+                        "target_min" : 1,
+                        "target_max" : 20
+                    },
+                    "miss_probability" : 0,
+                },
+            },
+            "constraints" : {
+                "custom_constraints": {
+                    "cc1" : {
+                        "filter"  : "B >= '2020/03/01'",
+                        "targets" : {
+                            "B, A" : "sort_and_skew_left",
+                        }
+                    },
+                }
+            },
+        }
+
+        test_data = pd.DataFrame(data={
+            "A" : test_dict["_rng"].normal(size=1000),
+            "B": pd.date_range(
+                    start="2020/01/01",
+                    periods=1000,
+                    freq="D",            
+                ),
+        })
+
+        # make sure A is all positive
+        test_data["A"] = test_data["A"] + abs(test_data["A"].min())
+        test_data["B"] = test_data["B"].sample(frac=1)
+
+        test_gen = tm.ConstraintHandler(test_dict, test_data)
+        result = test_gen.process_constraints()
+
+        self.assertGreater(
+            result.query("B >= '2020/03/01'")["A"].iloc[-188:].mean(),
+            result.query("B >= '2020/03/01'")["A"].iloc[:188].mean() * 2,
+        )
+
+    def test_custom_constraints_sort_and_skew_left_with_partition(self):
+        '''
+        Mean is to the left of the median. Making a column skewed requires at least
+        two target columns where one is the ordering column and another - whose 
+        distribution is being skewed.
+        '''
+
+        test_dict = {
+
+            "_rng" : np.random.default_rng(seed=0),
+            "columns" : {
+                "A" : {
+                    "type": "continuous",
+                    "precision" : "float",
+                    "distribution" : "weighted_uniform",
+                    "distribution_parameters": {
+                        "dispersion" : 0.1,
+                        "target_mean" : 25,
+                        "target_std" : 2
+                    },
+                    "miss_probability" : 0,
+                },
+            },
+            "constraints" : {
+                "custom_constraints": {
+                    "cc1" : {
+                        "filter"  : "B >= '2020/03/01'",
+                        "partition" : "C",
+                        "targets" : {
+                            "B, A" : "sort_and_skew_left",
+                        }
+                    },
+                }
+            },
+        }
+
+        test_data = pd.DataFrame(data={
+            "A" : test_dict["_rng"].normal(size=1000),
+            "B": pd.date_range(
+                    start="2020/01/01",
+                    periods=1000,
+                    freq="D",            
+                ),
+            "C" : ["spam", "ham"] * 500,
+        })
+
+        # make sure A is all positive
+        test_data["A"] = test_data["A"] + abs(test_data["A"].min())
+        test_data["B"] = test_data["B"].sample(frac=1)
+
+        test_gen = tm.ConstraintHandler(test_dict, test_data)
+        result = test_gen.process_constraints()
+
+        self.assertGreater(
+            result.query("(B >= '2020/03/01') & C == 'spam'")["A"].iloc[-188:].mean(),
+            result.query("(B >= '2020/03/01') & C == 'spam'")["A"].iloc[:188].mean() * 2
+        )
+
 if __name__ == "__main__" and __package__ is None:
     #overwrite __package__ builtin as per PEP 366
     __package__ = "exhibit"
